/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package multivarquestions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * This class is demo implementation in Java, to develop the algorithm for a
 * question answer matrix. A qa-matrix has the answers formatted as row header
 * elements, the row headers and the questions and the cells are the tick boxes.
 *
 * The implementation takes a number of <b>question-answer pair</b>s and
 * <b>dummy pair</b>s (which contain only wrong answers as a kind of decoy),
 * currently in the main method.
 *
 * To create an exam question proper,
 * <ol><li>first the pairs are shuffled.</li>
 * <li>Then for each of the pair, the answer is used as a header elements (top
 * cell). In this state, filling out the matrix would put the correct answers on
 * the diagonal, skipping the decoy answers.</li>
 *
 * <li>The rows are built separately, but only from the proper question-answer
 * pairs, not from the dummy pair. </li>
 * <li>For each pair the correct answer cell number can be determined from the
 * row number (remember diagonal). Thereby you are able to create the list of
 * right and wrong choices. Example: Pair two (of six ) would have .X...., where
 * '.' is wrong and 'X' right. Thus we can create the question rows.</li>
 * <li> If it is not a dummy pair, the row is added to the row-set.</li>
 * <li> After completing the adding of the questions to the row-set, the row-set
 * is shuffled and can then be added to output stream and printed.</li>
 * </ol>
 *
 *
 * @author hom
 */
public class MultiVarQuestions {

    private static Pair[] pairs =
            new Pair[]{
        new QAPair( "aNetherlands", "Amsterdam " ),
        new QAPair( "bBelgium    ", "Bruxelles " ),
        new QAPair( "cDanmark    ", "Copenhagen" ),
        new QAPair( "dIndia      ", "Dehli     " ),
        new QAPair( "eFrance     ", "EParis    " ),
        new Pair( "Not used    ", "FDublin   " ),
        new Pair( "Not used    ", "GLondon   " ),
        new QAPair( "Egypt       ", "Cario     " ),
        new QAPair( "Turkey      ", "Istanbul  " ),
        new QAPair( "Greece      ", "Athens    " ),
        new QAPair( "Spain       ", "Madrid    " ), };

    private static int countRows() {
        int result = 0;
        for ( Pair p : pairs ) {
            if ( ( p instanceof QAPair ) ) {
                result++;
            }
        }
        return result;
    }

    private static void printPairs(
            List<Pair> pList ) {
        for ( Pair p : pList ) {
            System.out.println( p );
        }
    }

    private static void printQuestions(
            List<String> header,
            List<Question> questions ) {
        System.out.println( "             " + header );
        for ( Question q : questions ) {
            System.out.println( q );
        }
    }
    private List<Pair> pairList;
    List<String> header;
    List<Question> questions;

    private void questionsAndHeaders(String wrong, String correct ) {
        int c = 0;
        header = new ArrayList<>();
        questions = new ArrayList<>();
        for ( Pair p : pairList ) {
            header.add( p.answer );
            if ( p instanceof QAPair ) {
                Question question = new Question( p.question, pairList.size() );
                question.markChoice( c, wrong, correct );
                questions.add( question );
            }
            c++;
        }
    }

    static class Pair {

        String question;
        String answer;

        public Pair( String question, String Solution ) {
            this.question = question;
            this.answer = Solution;
        }

        @Override
        public String toString() {
            return getClass().getSimpleName() + "\tq=" + question + ",\t a="
                    + answer;
        }
    }

    static class QAPair extends Pair {

        public QAPair( String question, String Solution ) {
            super( question, Solution );
        }
    };

    static class Question {

        String question;
        String[] choices;

        public Question( String question, int choiceCount ) {
            this.question = question;
            this.choices = new String[ choiceCount ];
        }

        Question markChoice( int m, String wrong, String correct ) {
            for ( int c = 0; c < choices.length; c++ ) {
                choices[c] = wrong;
            }
            choices[m] = correct;
            return this;
        }

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder( question ).append( " " );
            if ( choices.length > 0 ) {
                int c = 0;

                sb.append( "[     " ).append( choices[c] );
                c++;
                for ( ; c < choices.length; c++ ) {
                    sb.append( "    ,      " )
                            .append( choices[c] );
                }
            }
            sb.append( "    ]" );
            return sb.toString();
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main( String[] args ) {
        new MultiVarQuestions().demo();
    }

    public MultiVarQuestions() {
        pairList = Arrays.asList( pairs );
        rowCount = countRows();
    }
    int rowCount = countRows();

    void demo() {
        System.out.println( "As defined" );
        printPairs( pairList );
        System.out.println( "shuffled pairs" );
        Collections.shuffle( pairList );
        printPairs( pairList );

        questionsAndHeaders( " ","X" );
        System.out.println( "questions in start order" );
        System.out.println( "             "
                + "[01234567890123456789012346789012345789012345689012345678901234567890]" );
        printQuestions( header, questions );
        Collections.shuffle( questions );
        System.out.println( "questions after shuffle" );
        printQuestions( header, questions );

        System.out.println( "latex intermediate" );
        questionsAndHeaders( "\\wrongchoice{}  ",
                             "\\correctchoice{}" );
        Collections.shuffle( questions );
        printQuestions( header, questions );

    }
}
